/*   Данный скетч делает следующее: передатчик (TX) отправляет массив
     данных, который генерируется согласно показаниям с кнопки и с
     двух потенциомтеров. Приёмник (RX) получает массив, и записывает
     данные на реле, сервомашинку и генерирует ШИМ сигнал на транзистор.
    by AlexGyver 2016
*/

#include <SPI.h>          // библиотека для работы с шиной SPI
#include "nRF24L01.h"     // библиотека радиомодуля
#include "RF24.h"         // ещё библиотека радиомодуля
#include <Servo.h>

Servo myservo1;
Servo myservo2;
Servo myservo3;
Servo myservo4;
Servo myservo5;
Servo myservo6;

RF24 radio(9, 10); // "создать" модуль на пинах 9 и 10 Для Уно
//RF24 radio(9,53); // для Меги

byte address[][6] = {"1Node", "2Node", "3Node", "4Node", "5Node", "6Node"}; //возможные номера труб

byte counter;

int glove[8]; // массив данных для отправки
int manipulator[2]; // входящий массив от манипулятора (обратная тактильная связь)

void setup() {
  Serial.begin(9600);         // открываем порт для связи с ПК

  radio.begin();              // активировать модуль
  radio.setAutoAck(1);        // режим подтверждения приёма, 1 вкл 0 выкл
  radio.setRetries(0, 15);    // (время между попыткой достучаться, число попыток)
  radio.enableAckPayload();   // разрешить отсылку данных в ответ на входящий сигнал
  radio.setPayloadSize(32);   // размер пакета, в байтах

  radio.openWritingPipe(address[0]);// мы - труба 0, открываем канал для передачи данных
  radio.openReadingPipe(0, address[1]);  // + открываем чтение сигнала
  radio.setChannel(0x70);             // выбираем канал (в котором нет шумов!)

  radio.setPALevel (RF24_PA_MAX);   // уровень мощности передатчика. На выбор RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH, RF24_PA_MAX
  radio.setDataRate (RF24_2MBPS); // скорость обмена. На выбор RF24_2MBPS, RF24_1MBPS, RF24_250KBPS
  //должна быть одинакова на приёмнике и передатчике!
  //при самой низкой скорости имеем самую высокую чувствительность и дальность!!

  radio.powerUp();        // начать работу
  radio.startListening(); // начинаем слушать эфир

  Serial.println("Started");  // оповещаем оператара, что живы

  // прописываем пины, к которым подключены сервоприводы
  myservo1.attach(4);
  myservo2.attach(5);
  myservo3.attach(6);
  myservo4.attach(7);
  myservo5.attach(8);
  myservo6.attach(3);

  //  прописываем входы сигналов от потенцеометров с пальцев
  pinMode (A3, INPUT);
  pinMode (A4, INPUT);
  pinMode (A5, INPUT);
  pinMode (A6, INPUT);
  pinMode (A7, INPUT);
  // и с кистевого сустава
  pinMode (A0, INPUT);  // вращение предплечия вокруг своей оси
  pinMode (A1, INPUT);  // вращение кисти влево-вправо
  pinMode (A2, INPUT);  // и вверх-вниз
}

// прописываем переменные для чтения аналогово сигнала потенцеометров
int pot_1;
int pot_2;
int pot_3;
int pot_4;
int pot_5;

int pot_6;
int pot_7;
int pot_8;
// + переменные для полученя обратной связи
int signal_1;
int signal_2;

// + переменные буфера для остановки двигателей
int buf_1;
int buf_2;
int buf_3;
int buf_4;
int buf_5;

void loop() {
  byte pipeNo, gotByte; // прописываем, что хотим слушать любую трубу

  //  читаем сигнал потенцеометров
  pot_1 = map(analogRead(A3), 0, 1023, 300, 0);
  pot_2 = map(analogRead(A4), 0, 1023, 300, 0);
  pot_3 = map(analogRead(A5), 0, 1023, 300, 0);
  pot_4 = map(analogRead(A7), 0, 1023, 300, 0);
  pot_5 = map(analogRead(A6), 0, 1023, 300, 0);

  pot_6 = map(analogRead(A0), 1023, 700, 180, 0);
  pot_7 = map(analogRead(A1), 1023, 0, 300, 0);
  pot_8 = map(analogRead(A2), 1023, 0, 300, 0);

  // радуем оператора, тем, что всё ещё живы
  Serial.print(190); Serial.print(" ");
  Serial.print(-10); Serial.print(" ");
  Serial.print(pot_1); Serial.print(" ");
  Serial.print(pot_2); Serial.print(" ");
  Serial.print(pot_3); Serial.print(" ");
  Serial.print(pot_4); Serial.print(" ");
  Serial.print(pot_5); Serial.print(" ");
  Serial.print(pot_6); Serial.print(" ");
  Serial.print(pot_7); Serial.print(" ");
  Serial.print(pot_8); Serial.println(" ");

  // проверяем: хочет ли оператор надеть перчатк

  // защита сервоприводов от недопонимания
  if (pot_1 > 179){pot_1 = 180;}
  if (pot_2 > 179){pot_2 = 180;}
  if (pot_3 > 179){pot_3 = 180;}
  if (pot_4 > 179){pot_4 = 180;}
  if (pot_5 > 179){pot_5 = 180;}

  if (pot_6 > 179){pot_6 = 180;}
  if (pot_7 > 179){pot_7 = 180;}
  if (pot_8 > 179){pot_8 = 180;}

  if (pot_1 < 1){pot_1 = 1;}
  if (pot_2 < 1){pot_2 = 1;}
  if (pot_3 < 1){pot_3 = 1;}
  if (pot_4 < 1){pot_4 = 1;}
  if (pot_5 < 1){pot_5 = 1;}

  if (pot_6 < 1){pot_6 = 1;}
  if (pot_7 < 1){pot_7 = 1;}
  if (pot_8 < 1){pot_8 = 1;}

  // заполняем массив для отправки
  glove[0] = pot_1;
  glove[1] = pot_2;
  glove[2] = pot_3;
  glove[3] = pot_4;
  glove[4] = pot_5;

  glove[5] = pot_6;
  glove[6] = pot_7;
  glove[7] = pot_8;

  // корректируем обратную тактильную связь
  pot_1 += 15;
  pot_2 -= 13;
  pot_3 += 15;
  pot_4 -= 5;

  // больше защиты богу защиты
  if (pot_1 > 179){pot_1 = 180;}
  if (pot_3 > 179){pot_3 = 180;}

  if (pot_2 < 1){pot_2 = 1;}
  if (pot_4 < 1){pot_4 = 1;}

  // инвертируем нужные сервы (из-за конструкции)
  pot_5 = map(pot_5, 0, 180, 180, 0);

  // при поступлении сигнала, блокируем сервопривод
  if (signal_1 > 0){pot_1 = buf_1;}
  if (signal_2 > 0){pot_2 = buf_2; pot_3 = buf_3; pot_4 = buf_4; pot_5 = buf_5;}


  // посылаем сигнал на сервоприводы
  myservo1.write(pot_1);
  myservo2.write(pot_2);
  myservo3.write(pot_3);
  myservo4.write(pot_4);
  myservo5.write(pot_5);
  myservo6.write(30);

  radio.stopListening(); // перестаём слушать эфир
  radio.write(&glove, sizeof(glove)); // отправляем данные
  radio.startListening(); // начинаем слушать эфир
  
  signal_1 = 0;
  signal_2 = 0;


  if (radio.available(&pipeNo)) {  // слушаем эфир со всех труб
    radio.read(&manipulator, sizeof(manipulator));  // чиатем входящий сигнал

    // заполняем переменные с помощью принятого массива
    signal_1 = manipulator[1];
    signal_2 = manipulator[0];


    // радуем оператора, тем, что всё ещё живы
    Serial.print("Принято: "); Serial.println(" ");
    Serial.print(signal_1); Serial.print(" ");
    Serial.print(signal_2); Serial.println(" ");
  }

  // обновляем буфер если можно
  if (signal_1 < 1){buf_1 = pot_1;}
  if (signal_2 < 1){buf_2 = pot_2; buf_3 = pot_3; buf_4 = pot_4; buf_5 = pot_5;}

  delay(50); // задержка для корректной работы сервоприводов
}
