#include <SPI.h>          // библиотека для работы с шиной SPI
#include "nRF24L01.h"     // библиотека радиомодуля
#include "RF24.h"         // ещё библиотека радиомодуля
#include <Servo.h>

RF24 radio(9, 10); // "создать" модуль на пинах 9 и 10 Для Уно
//RF24 radio(9,53); // для Меги

byte address[][6] = {"1Node", "2Node", "3Node", "4Node", "5Node", "6Node"}; //возможные номера труб

byte counter;

int glove[8]; // входящий массив от перчатки
int manipulator[2]; // массив для отправки (обратная тактильная связь)

// прописываем, какие сервы хотим использывать
Servo servo_1;
Servo servo_2;
Servo servo_3;
Servo servo_4;
Servo servo_5;

void setup() {
  Serial.begin(9600);         // открываем порт для связи с ПК

  radio.begin();              // активировать модуль
  radio.setAutoAck(1);        // режим подтверждения приёма, 1 вкл 0 выкл
  radio.setRetries(0, 15);    // (время между попыткой достучаться, число попыток)
  radio.enableAckPayload();   // разрешить отсылку данных в ответ на входящий сигнал
  radio.setPayloadSize(32);   // размер пакета, в байтах

  radio.openWritingPipe(address[1]);  // мы - труба 1, открываем канал для передачи данных
  radio.openReadingPipe(1, address[0]);  // + открываем чтение сигнала
  radio.setChannel(0x70);             // выбираем канал (в котором нет шумов!)

  radio.setPALevel (RF24_PA_MAX);   // уровень мощности передатчика. На выбор RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH, RF24_PA_MAX
  radio.setDataRate (RF24_2MBPS); // скорость обмена. На выбор RF24_2MBPS, RF24_1MBPS, RF24_250KBPS
  //должна быть одинакова на приёмнике и передатчике!
  //при самой низкой скорости имеем самую высокую чувствительность и дальность!!

  radio.powerUp();        // начать работу
  radio.startListening();  // начинаем слушать эфир

  Serial.println("Started");  // оповещаем оператара, что живы

  // прописываем пины, к которым подключены сервоприводы
  servo_1.attach(3);
  servo_2.attach(A0);
  servo_3.attach(4);
  servo_4.attach(5);
  servo_5.attach(6);

  // прописываем пины концевиков
  pinMode(7, INPUT);
  pinMode(8, INPUT);

}

// переменные для чтения принятого сигнала 
// (можно всё время обращаться к массиву, но это выгдядит не удобно)
int signal_1;
int signal_2;
int signal_3;
int signal_4;
int signal_5;
int signal_6;
int signal_7;
int signal_8;

// переменные для считывания сигнала концевиков (обратная связь)
int but_1;
int but_2;

void loop() {
  byte pipeNo, gotByte; // прописываем, что хотим слушать любую трубу

  // чипаем сигналы с концевиков и заполняем переменные
  but_1 = digitalRead(7);
  but_2 = digitalRead(8);

  if (radio.available(&pipeNo)) {        // слушаем эфир со всех труб
    radio.read(&glove, sizeof(glove));  // чиатем входящий сигнал

    // заполняем переменные с помощью принятого массива
    signal_1 = map(glove[0], 0, 60, 0, 90);
    signal_2 = glove[1];
    signal_3 = glove[2];
    signal_4 = glove[3];
    signal_5 = glove[4];
    signal_6 = glove[5];
    signal_7 = map(glove[6], 0, 180, 180, 0);
    signal_8 = map(glove[7], 0, 180, 0, 80);

    // радуем оператора, тем, что всё ещё живы
    Serial.print(signal_1); Serial.print(" ");
    Serial.print(signal_2); Serial.print(" ");
    Serial.print(signal_3); Serial.print(" ");
    Serial.print(signal_4); Serial.print(" ");
    Serial.print(signal_5); Serial.print(" ");
    Serial.print(signal_6); Serial.print(" ");
    Serial.print(signal_7); Serial.print(" ");
    Serial.print(signal_8); Serial.print(" ");
    Serial.print(but_1); Serial.print(" ");
    Serial.print(but_2); Serial.println(" ");

    // посылаем, принятый сигнал на сервоприводы
    int signal = (signal_2 + signal_3 + signal_4 + signal_5) / 4;
    signal = map (signal, 0, 140, 0, 90);
    servo_1.write(signal_6);
    servo_2.write(signal_8);
    servo_3.write(signal_7);
    servo_4.write(signal);
    servo_5.write(signal_1);
  }

  if (but_1 == 1 or but_2 == 1){  // проверяем нажат ли концевик
    // обнуляем массив
    manipulator[0] = 0;
    manipulator[1] = 0;
    
    // заполняем массив при надобности
    if (but_1 == 1){manipulator[0] = 100;}
    if (but_2 == 1){manipulator[1] = 100;}

    Serial.print(but_1); Serial.print(" ");
    Serial.print(but_2); Serial.println(" ");

    radio.stopListening(); // перестаём слушать эфир
    radio.write(&manipulator, sizeof(manipulator)); // отправляем данные
    radio.startListening(); // начинаем слушать эфир
  }

  delay(10);
}
